﻿using System.Net;

namespace SddpViewer;

using System;
using System.Threading.Tasks;
using CommunityToolkit.Mvvm.ComponentModel;
using Rssdp;

public partial class DiscoveredDeviceViewModel : ObservableObject
{
  private readonly DiscoveredSsdpDevice _device;
  private SsdpDevice? _ssdpDevice;

  public DiscoveredDeviceViewModel(DiscoveredSsdpDevice device)
  {
    _device = device;
    ResponseHeader = GetResponseHeader();
    DiscoveredAt = DateTime.Now;
    IpAddress = EvaluateIpAddress(device);
    MacAddress = EvaluateMacAddress();
    HostName = EvaluateHostName();
  }

  public string HostName { get; set; }

  public string MacAddress { get; set; }

  public string IpAddress { get; set; }

  public DateTime DiscoveredAt { get; }

  [ObservableProperty]
  public partial string ResponseHeader { get; set; }

  /// <summary>
  /// Sets or returns the type of notification, being either a uuid, device type, service type or upnp:rootdevice.
  /// </summary>
  public string NotificationType => _device.NotificationType;

  /// <summary>
  /// Sets or returns the universal service name (USN) of the device.
  /// </summary>
  public string Usn => _device.Usn;

  /// <summary>
  /// Sets or returns a URL pointing to the device description document for this device.
  /// </summary>
  public Uri DescriptionLocation => _device.DescriptionLocation;

  /// <summary>
  /// Sets or returns the length of time this information is valid for (from the <see cref="P:Rssdp.DiscoveredSsdpDevice.AsAt" /> time).
  /// </summary>
  public TimeSpan CacheLifetime => _device.CacheLifetime;

  /// <summary>
  /// Sets or returns the date and time this information was received.
  /// </summary>
  public DateTimeOffset AsAt => _device.AsAt;

  [ObservableProperty]
  public partial string FriendlyName { get; set; } = "";

  [ObservableProperty]
  public partial SsdpDeviceIcon? Icon { get; set; }

  [ObservableProperty]
  public partial Uri? PresentationUrl { get; set; }

  [ObservableProperty]
  public partial string ModelNumber { get; set; } = "";

  [ObservableProperty]
  public partial string Version { get; set; } = "";

  public async Task GetFurtherInformationAsync()
  {
    _ssdpDevice = await _device.GetDeviceInfo().ConfigureAwait(false);
    FriendlyName = _ssdpDevice.ModelDescription;
    Icon = _ssdpDevice.Icons.MinBy(x => x.Height);
    PresentationUrl = _ssdpDevice.PresentationUrl;
    ModelNumber = _ssdpDevice.ModelNumber;
    Version = _ssdpDevice.SerialNumber?.Split(',').Last() ?? new Version().ToString();
    HttpClient client = new HttpClient();
    var response = await client.GetAsync(_ssdpDevice.ModelUrl).ConfigureAwait(false);
    ResponseHeader = await response.Content.ReadAsStringAsync();
  }

  private string EvaluateHostName() => Dns.GetHostEntry(IpAddress).HostName;

  private string GetResponseHeader() =>
    string.Join("," + Environment.NewLine, _device.ResponseHeaders.Select(x => $"{{{x.Key} : {string.Join(";", x.Value)}}}"));

  private string EvaluateIpAddress(DiscoveredSsdpDevice device) => device.DescriptionLocation.Host;

  private string EvaluateMacAddress()
  {
    var lookupResult = ArpLookup.Arp.Lookup(IPAddress.Parse(IpAddress));
    return lookupResult is null ? "Unknown" : string.Join(":", lookupResult.GetAddressBytes().Select(b => $"{b:x2}"));
  }

  public override bool Equals(object? obj)
  {
    return obj is DiscoveredDeviceViewModel viewModel && Equals(viewModel.Usn, Usn);
  }
}
