﻿using System.Collections.Specialized;

namespace SddpViewer;

using System.Collections.ObjectModel;
using System.Threading.Tasks;
using Avalonia.Threading;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Rssdp;

public sealed partial class MainWindowViewModel : ObservableObject, IDisposable
{
  public MainWindowViewModel()
  {
    NetworkAdapters = NetworkAdapter.GetAvailableNetworkAdapter().ToArray();
    SelectedNetworkAdapter = NetworkAdapters.FirstOrDefault();
    SddpDevices = new ObservableCollection<DiscoveredDeviceViewModel>();
    SddpDevices.CollectionChanged += SddpDevices_OnCollectionChanged;
  }

  private static async void SddpDevices_OnCollectionChanged(object? sender, NotifyCollectionChangedEventArgs e)
  {
    try
    {
      foreach (object eNewItem in e.NewItems ?? Array.Empty<object>())
      {
        if (eNewItem is DiscoveredDeviceViewModel discoveredDeviceViewModel)
        {
          await discoveredDeviceViewModel.GetFurtherInformationAsync();
        }
      }
    }
    catch (Exception ex)
    {
      throw; // TODO handle exception
    }
  }

  private void LocatorOnDeviceUnavailable(object? sender, DeviceUnavailableEventArgs e)
  {
    var existingDevice = SddpDevices.FirstOrDefault(x => string.Equals(x.Usn, e.DiscoveredDevice.Usn, StringComparison.Ordinal));
    if (existingDevice is null)
      return;
    Dispatcher.UIThread.Invoke(() => SddpDevices.Remove(existingDevice));
  }

  private void LocatorOnDeviceAvailable(object? sender, DeviceAvailableEventArgs e)
  {
    if (!e.IsNewlyDiscovered)
    {
      return;
    }

    var discoveredDeviceViewModel = new DiscoveredDeviceViewModel(e.DiscoveredDevice);
    if (!SddpDevices.Contains(discoveredDeviceViewModel))
      Dispatcher.UIThread.Invoke(() => SddpDevices.Add(discoveredDeviceViewModel));
  }

  [ObservableProperty]
  private IReadOnlyList<NetworkAdapter> _networkAdapters;

  [ObservableProperty]
  private NetworkAdapter? _selectedNetworkAdapter;

  [ObservableProperty]
  private string _deviceIpAddress = "192.168.42.193";

  [ObservableProperty]
  private string _notificationFilter = "upnp:rootdevice";

  private SsdpDeviceLocator? _locator;

  public ObservableCollection<DiscoveredDeviceViewModel> SddpDevices { get; }

  [RelayCommand]
  private async Task SearchDevicesNowAsync()
  {
    SddpDevices.Clear();
    if (_locator is not null)
    {
      _locator.StopListeningForNotifications();
      _locator.DeviceAvailable -= LocatorOnDeviceAvailable;
      _locator.DeviceUnavailable -= LocatorOnDeviceUnavailable;
    }

    await StartListening(async locator =>
    {
      foreach (var discoveredSsdpDevice in await locator.SearchAsync())
      {
        var discoveredDeviceViewModel = new DiscoveredDeviceViewModel(discoveredSsdpDevice);
        Dispatcher.UIThread.Invoke(() => SddpDevices.Add(discoveredDeviceViewModel));
      }
    });
  }

  [RelayCommand]
  private async Task ResearchAsync()
  {
    if (_locator is not null)
    {
      await _locator.SearchAsync();
    }
  }

  [RelayCommand]
  private async Task StartListening(Func<SsdpDeviceLocator, Task>? action = null)
  {
    _locator = new SsdpDeviceLocator(SelectedNetworkAdapter?.IpAddress.ToString());
    if (!string.IsNullOrWhiteSpace(NotificationFilter))
    {
      _locator.NotificationFilter = NotificationFilter;
    }

    if (action is not null)
    {
      await action.Invoke(_locator);
    }

    _locator.DeviceAvailable += LocatorOnDeviceAvailable;
    _locator.DeviceUnavailable += LocatorOnDeviceUnavailable;
    _locator.StartListeningForNotifications();
  }

  public void Dispose()
  {
    _locator?.Dispose();
  }
}
